// Define the data for the tree
const treeData = {
  name: 'Netflix Catalog',
  children: [
    {
      name: 'Dramas',
      children: [
        {
          name: 'Movies',
          children: [
            { name: 'Sankofa' },
            { name: 'Je Suis Karl' },
          ],
        },
        {
          name: 'TV Shows',
          children: [
            { name: 'The Crown' },
            { name: 'Breaking Bad' },
          ],
        },
      ],
    },
    {
      name: 'Comedies',
      children: [
        {
          name: 'Movies',
          children: [
            { name: 'The Starling' },
            { name: 'Avvai Shanmughi' },
          ],
        },
        {
          name: 'TV Shows',
          children: [
            { name: 'Brooklyn Nine-Nine' },
            { name: 'The Good Place' },
          ],
        },
      ],
    },
    {
      name: 'Action & Adventure',
      children: [
        {
          name: 'Movies',
          children: [
            { name: 'The Stronghold' },
            { name: 'Birth of the Dragon' },
          ],
        },
        {
          name: 'TV Shows',
          children: [
            { name: 'Arrow' },
            { name: 'Daredevil' },
          ],
        },
      ],
    },
  ],
};

// Set the dimensions and margins of the diagram
const margin = {
  top: 20,
  right: 120,
  bottom: 20,
  left: 120,
};
const width = 960 - margin.right - margin.left;
const height = 800 - margin.top - margin.bottom;

// Append the svg object to the body of the page
const svg = d3
  .select('body')
  .append('svg')
  .attr('width', width + margin.right + margin.left)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',' + margin.top + ')',
  );

// Create the tree layout
const treemap = d3.tree().size([height, width]);

// Assigns the data to a hierarchy using parent-child relationships
let nodes = d3.hierarchy(treeData, function (d) {
  return d.children;
});

// Maps the node data to the tree layout
nodes = treemap(nodes);

// Adds the links between the nodes
const link = svg
  .selectAll('.link')
  .data(nodes.descendants().slice(1))
  .enter()
  .append('path')
  .attr('class', 'link')
  .attr('d', function (d) {
    return (
      'M' +
      d.y +
      ',' +
      d.x +
      'C' +
      (d.y + d.parent.y) / 2 +
      ',' +
      d.x +
      ' ' +
      (d.y + d.parent.y) / 2 +
      ',' +
      d.parent.x +
      ' ' +
      d.parent.y +
      ',' +
      d.parent.x
    );
  });

// Adds each node as a group
const node = svg
  .selectAll('.node')
  .data(nodes.descendants())
  .enter()
  .append('g')
  .attr('class', function (d) {
    return (
      'node' +
      (d.children ? ' node--internal' : ' node--leaf')
    );
  })
  .attr('transform', function (d) {
    return 'translate(' + d.y + ',' + d.x + ')';
  });

// Adds the circle to the node
node
  .append('circle')
  .attr('r', 10)
  .style('stroke', 'steelblue')
  .style('fill', 'lightsteelblue');

// Adds the text to the node
node
  .append('text')
  .attr('dy', '.35em')
  .attr('x', function (d) {
    return d.children ? -13 : 13;
  })
  .style('text-anchor', function (d) {
    return d.children ? 'end' : 'start';
  })
  .text(function (d) {
    return d.data.name;
  });
